﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Reflection;
using System.Drawing;
using System.Security.Cryptography;

namespace ROTools.ROWebLogin {

    public class AppConfigData {

        /// <summary>
        /// ガンホーIDデータリスト
        /// </summary>
        public List<GunghoAccountData> GunghoAccounts {
            get;
            set;
        }

        /// <summary>
        /// メインフォームの場所
        /// </summary>
        public Point MainFormLocaiton {
            get;
            set;
        }

        /// <summary>
        /// メインフォームのサイズ
        /// </summary>
        public Size MainFormSize {
            get;
            set;
        }

        /// <summary>
        /// Urdrワールド選択状態
        /// </summary>
        public bool IsUrdrWorld {
            get;
            set;
        }

        /// <summary>
        /// 設定暗号化
        /// </summary>
        public bool IsConfigCrypto {
            get;
            set;
        }

        /// <summary>
        /// パッチクライアント実行時の処理
        /// </summary>
        public string OnPatchClientExecOperate {
            get;
            set;
        }

        /// <summary>
        /// ゲームクライアント終了時の処理
        /// </summary>
        public string OnGameClientTerminateOperate {
            get;
            set;
        }

        /// <summary>
        /// 自動管理者権限実行
        /// </summary>
        public bool IsAutoAdminExec {
            get;
            set;
        }

        /// <summary>
        /// RagPath指定
        /// </summary>
        public string ExplicitRagPath {
            get;
            set;
        }

        /// <summary>
        /// 自動ログアウト実行
        /// </summary>
        public bool IsAutoLogoutExec {
            get;
            set;
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public AppConfigData() {
            GunghoAccounts = null;
            MainFormLocaiton = new Point();
            MainFormSize = new Size();
            IsUrdrWorld = false;
            IsConfigCrypto = false;
            OnPatchClientExecOperate = null;
            OnGameClientTerminateOperate = null;
            IsAutoAdminExec = true;
            ExplicitRagPath = null;
            IsAutoLogoutExec = false;
        }



        /// <summary>
        /// 保存用ファイル名の取得
        /// </summary>
        /// <returns></returns>
        private static string GetBaseFileName() {
            // 実行ファイル名を取得する
            string exeFileName = Assembly.GetExecutingAssembly().Location;
            int periodPos = exeFileName.LastIndexOf('.');
            if(periodPos <= 0) {
                throw new Exception("実行ファイル名が正しく取得できませんでした");
            }
            return exeFileName.Substring(0, periodPos);
        }


        /// <summary>
        /// XML形式でのシリアライズ
        /// </summary>
        /// <param name="stream"></param>
        private void XmlSerialize(Stream stream) {
            XmlSerializer serializer = new XmlSerializer(typeof(AppConfigData));
            serializer.Serialize(stream, this);
        }


        /// <summary>
        /// XML形式からのデシリアライズ
        /// </summary>
        /// <param name="strema"></param>
        /// <returns></returns>
        private static AppConfigData XmlDeserialize(Stream stream) {
            XmlSerializer serializer = new XmlSerializer(typeof(AppConfigData));
            return (AppConfigData)serializer.Deserialize(stream);
        }


        /// <summary>
        /// XMLファイルへ出力する
        /// </summary>
        public void XmlFileSerialize() {
            XmlFileSerialize(GetBaseFileName() + ".xml");
        }

        /// <summary>
        /// XMLファイルへ出力する
        /// </summary>
        public void XmlFileSerialize(string fileName) {
            using(Stream stream = new FileStream(fileName, FileMode.Create)) {
                XmlSerialize(stream);
            }
        }

        /// <summary>
        /// XMLファイルから読み込む
        /// </summary>
        /// <returns></returns>
        public static AppConfigData XmlFileDeserialize() {
            return XmlFileDeserialize(GetBaseFileName() + ".xml");
        }

        /// <summary>
        /// XMLファイルから読み込む
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public static AppConfigData XmlFileDeserialize(string fileName) {
            using(Stream stream = new FileStream(fileName, FileMode.Open)) {
                return AppConfigData.XmlDeserialize(stream);
            }
        }

        /// <summary>
        /// 暗号化して出力する
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="key"></param>
        private void CryptoSerialize(Stream stream, string key) {
            DESCryptoServiceProvider desProvider = CryptoUtil.CreateProvider(key);
            using(ICryptoTransform cryptor = desProvider.CreateEncryptor()) {
                using(CryptoStream cs = new CryptoStream(stream, cryptor, CryptoStreamMode.Write)) {
                    XmlSerialize(cs);
                }
            }
        }

        /// <summary>
        /// 復号化して復元する
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="key"></param>
        /// <returns></returns>
        public static AppConfigData CryptoDeserialize(Stream stream, string key) {
            DESCryptoServiceProvider desProvider = CryptoUtil.CreateProvider(key);
            using(ICryptoTransform cryptor = desProvider.CreateDecryptor()) {
                using(CryptoStream cs = new CryptoStream(stream, cryptor, CryptoStreamMode.Read)) {
                    return XmlDeserialize(cs);
                }
            }
        }


        /// <summary>
        /// 暗号化してファイルへ出力する
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="key"></param>
        public void CryptoFileSerialize(string key) {
            CryptoFileSerialize(GetBaseFileName() + ".dat", key);
        }

        /// <summary>
        /// 暗号化してファイルへ出力する
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="key"></param>
        public void CryptoFileSerialize(string fileName, string key) {
            using(Stream stream = new FileStream(fileName, FileMode.Create)) {
                CryptoSerialize(stream, key);
            }
        }


        /// <summary>
        /// ファイルから復号化して復元する
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static AppConfigData CryptoFileDeserialize(string key) {
            return CryptoFileDeserialize(GetBaseFileName() + ".dat", key);
        }

        /// <summary>
        /// ファイルから復号化して復元する
        /// </summary>
        /// <param name="fileName"></param>
        /// <param name="key"></param>
        /// <returns></returns>
        public static AppConfigData CryptoFileDeserialize(string fileName, string key) {
            using(Stream stream = new FileStream(fileName, FileMode.Open)) {
                return CryptoDeserialize(stream, key);
            }
        }

    }

}
